import os
import cv2
import math
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit


PI=3.1419


def distance(p1,p2):
	diffX=p1[0]-p2[0]
	diffY=p1[1]-p2[1]
	return math.sqrt(diffY*diffY+diffX*diffX)



def func_chirp(dis,a,b,c):
	return (a*dis+b)/(c*dis+1)

def invert_chirp(a,b,c,y):
	return int((y-b)/(a-y*c)*10)/float(10)

def cal_chirp(distances):
	if len(distances)<3:
		return 0,0
	step_ids=range(len(distances)+1)[1:]
	popt, pcov = curve_fit(func_chirp, distances, step_ids)
	a=popt[0]
	b=popt[1]
	c=popt[2]
	# print distances
	generated=[invert_chirp(a,b,c,id) for id in step_ids]
	# print generated
	# plt.scatter(distances,step_ids,c='b')
	# plt.scatter(generated,step_ids,c='r')
	# plt.show()

	t_chirps=[generated[0]-generated[1]]
	for id in range(1,len(generated)):
		t_chirps.append(generated[id-1]-generated[id])

	error_arr=[abs(dis-gen)/t for dis,gen,t in zip(distances,generated,t_chirps)]
	# print error_arr
	# print t_chirps
	average_error=np.mean(error_arr)
	# print average_error

	# print [int(func_chirp(dist,popt[0],popt[1],popt[2])) for dist in distances]
	# print (b*c)/a
	phi=math.atan(math.sqrt(abs((b*c)/a)))*180/PI
	z1=math.sqrt(abs(b/(a*c)))
	z2=math.sqrt(abs(a*b/c))
	scale=z1/z2

	return scale,phi,average_error,a,b,c

stair_error=[]
stair_params=[]



for _,_,file_lists in os.walk('./points/'):
	continue



# detect points on image

for i in range(len(file_lists)):
	img=cv2.imread('./points/'+file_lists[i])
	img_gray=cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
	img_bin=cv2.inRange(img_gray,80,255,cv2.THRESH_BINARY)


	pts=[]
	_,contours,_ = cv2.findContours(img_bin.copy(), cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)


	for c in contours:

		r=cv2.boundingRect(c)
		# print r
		pts.append((r[0]+r[2]/2,r[1]+r[3]/2))


	stair_distances=[int(distance(p1,pts[-1])) for p1 in pts[:-1]]
	s,phi,err,a,b,c=cal_chirp(stair_distances)

	# print s,phi,err

	stair_error.append(err*100)
	stair_params.append( [s,phi])

	print pts[-1]
	print a,b,c
	plt.imshow(img_bin)
	plt.show()
	break

stair_params=np.array(stair_params)




# plt.subplot(2,2,1)
# print len(stair_error)
# print np.mean(stair_error)
# plt.hist(stair_error)
# plt.grid()
# plt.xlabel('error (%)')
# plt.ylabel('number of viewpoints')
# # plt.show()


# plt.subplot(2,2,2)
# for i in range(12):
# 	plt.scatter(((stair_params[:,0])[i:])[::12],((stair_params[:,1])[i:])[::12],c=((np.ones(len(stair_params))[i:])[::12])*i*30,s=50,vmin=0,vmax=360)
# plt.title("Staircase viewpoint angles distribution")
# plt.grid()
# plt.colorbar()


# plt.subplot(2,2,3)
# for i in range(9):
# 	plt.scatter((stair_params[:,0])[i*9:(i+1)*9],(stair_params[:,1])[i*9:(i+1)*9],c=(np.ones(len((stair_params[:,1])[i*9:(i+1)*9])))*i,s=50,vmin=0,vmax=9)
# plt.title("Staircase viewpoint distances distribution")
# plt.colorbar()
# plt.xlabel('relative scale')
# plt.ylabel('angle (degree)')
# plt.grid()

# plt.show()
